<?php
/**
 * @version		$Id: curl.php 20196 2011-01-26 12:42:15Z donpool $
 * @package		JeTs.Framework
 * @subpackage	JcURL
 * @copyright	Copyright (C) 2006 - 2011 NuestroServer. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('JPATH_BASE') or die;

/**
 * Example - HowTo use it!
 *
	jimport('easyteck.curl.curl');
	
	$url = 'http://wlib.inpc.gob.ec/locations.php';
	$curl = new cURL( $url );
	$data = array( 
		'type'		=> 'parr',
		'provId'	=> '6',
		'cntnId'	=> '5'
	);
	$curl->get( $data );
	
	$data = json_decode( $curl->data );
	
	print_r( $data );
 * 
 */

class JcURL extends JObject {
	
	protected	$url	= null;
	var				$data	= null;
	var				$info	= null;
	
	function __construct($url) {
		$this->url = $url;
	}

	/** 
	 * Send a POST requst using cURL 
	 * and sets class vars $data and $info
	 * 
	 * @param array $data values to send 
	 * @param array $options for cURL 
	 * @return true 
	 */ 
	function post(array $data = NULL, array $options = array()) 
	{ 
		$defaults = array( 
			CURLOPT_POST => 1, 
			CURLOPT_HEADER => 0, 
			CURLOPT_URL => $this->url, 
			CURLOPT_FRESH_CONNECT => 1, 
			CURLOPT_RETURNTRANSFER => 1, 
			CURLOPT_FORBID_REUSE => 1, 
			CURLOPT_TIMEOUT => 4, 
			CURLOPT_POSTFIELDS => http_build_query($data) 
		); 
		
		$this->exec($defaults, $options);
		
		return true; 
	} 
	
	/** 
	 * Send a GET requst using cURL 
	 * and sets class vars $data and $info
	 * 
	 * @param array $data values to send 
	 * @param array $options for cURL 
	 * @return true 
	 */ 
	function get(array $data = NULL, array $options = array()) 
	{    
		$defaults = array( 
			CURLOPT_URL => $this->url . (strpos($this->url, '?') === FALSE ? '?' : '') . http_build_query($data), 
			CURLOPT_HEADER => 0, 
			CURLOPT_RETURNTRANSFER => TRUE, 
			CURLOPT_TIMEOUT => 4 
		);
		$this->exec($defaults, $options);
		
		return true; 
	} 

	/**
	 * Calls a URL using cURL 
	 * and sets class vars $data and $info
	 *
	 * @param array $options for cURL
	 * @return true
	 */
	function call( array $options = array() ) {
		$defaults = array(
			CURLOPT_URL => $this->url, 			
			CURLOPT_HEADER => 0
		);
		$this->exec($defaults, $options);
		
		return true;
	}
	
	/**
	 * Calls a URL using cURL and loads the URL's content
	 * and sets class vars $data and $info
	 *
	 * @param array $options for cURL
	 * @return true
	 */
	function load( array $options = array() ) {
		$defaults = array(
			CURLOPT_URL => $this->url, 			
			CURLOPT_HEADER => 0,
			CURLOPT_RETURNTRANSFER => true
		);
		$this->exec($defaults, $options);
		
		return true;
	}

	/**
	 * Execs the cURL task and sets class vars $data and $info accordingly
	 *
	 * @param array $defaults options for cURL
	 * @param array $options (additional) for cURL
	 * @return true
	 */
	private function exec(array $defaults = array(), array $options = array()) {
		$ch = curl_init();
		curl_setopt_array($ch, ($options + $defaults)); 
		$this->data = curl_exec($ch);
		$this->info = curl_getinfo($ch);
		if( ! $this->data ) { 
			trigger_error(curl_error($ch)); 
		} 
		curl_close($ch);
		
		return true;
	}
	
//	function post( $data, $url ) {
//		/* http://localhost/upload.php:
//		print_r($_POST);
//		print_r($_FILES);
//		*/
//		
//		$ch = curl_init();
//		
//		// $data = array('name' => 'Foo', 'file' => '@/home/user/test.png');
//		
//		curl_setopt($ch, CURLOPT_URL, $url);
//		curl_setopt($ch, CURLOPT_POST, 1);
//		curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
//		
//		$retval = curl_exec($ch);
//		curl_close($ch);
//		
//		return $retval;
//	}
	
}	// End class